#! /bin/sh
# Surveillance du temps de connexion des enfants
# Vincent MAGNIN, 1er juin 2014, mis à jour le 26/06/2014
# 1 période = 5 minutes, peut être réglée en utilisant : crontab -e

# Variables d'environnement indispensables :
# pour notify-send :
dbus=$(ps -u enfants e | grep -Eo 'dbus-daemon.*address=unix:abstract=/tmp/dbus-[A-Za-z0-9]{10}' | tail -c35)
export DBUS_SESSION_BUS_ADDRESS=$dbus
# pour espeak :
export DISPLAY=:0

#----------------------------
# Déclaration des fonctions :
#----------------------------
# Cette fonction affiche un message grâce au système de notification
# et le prononce à haute voix grâce au synthétiseur vocal espeak
avertir ()
{
    local titre="$1"
    local corps="$2"
    local icone=$3
    
    notify-send "<big>$titre</big>" "$corps" --icon=$icone -t 10000
    espeak "$titre" -v french -s 130 -p 80
    sleep 3
}

# Cette fonction surveille et limite la durée d'utilisation d'un processus
# Un fichier log est généré avec l'heure inscrite toutes les cinq minutes
surveiller ()
{
    local processus=$1
    local limite=$2
    local en_cours
    local periodes_processus
    local fichier
    
#    en_cours=$(ps --no-headers -C $processus)
    en_cours=$(ps -u enfants | grep $processus)
    if [ "$en_cours" != "" ]; then
        fichier=$repertoire$jour$processus".log"
        echo $(date +%X) >> $fichier
        periodes_processus=$(wc -l < $fichier)
        if [ $periodes_processus -ge $limite ] ; then
            avertir "Fini de jouer à "$processus" !" "Je l'arrête !" face-angry
            pkill $processus
        elif [ $periodes_processus -ge $(($limite-1)) ] ; then
            avertir "Vous jouez trop à "$processus", encore cinq minutes !" "Il faut arrêter !" face-angry
        fi
    fi
}

#---------------------
# Programme principal
#---------------------
# On détermine le nom du fichier log d'aujourd'hui :
repertoire="/home/enfants/.parents/"
jour=$(date --rfc-3339=date)
fichier=$repertoire$jour".log"

# Lors de la première connexion de la journée (pas de fichier log) :
if [ ! -f $fichier ] ; then
    avertir "Bonjour les enfants !" "$(date)" face-angel
fi

# Evénements particuliers à signaler :
heures=$(date +%H)
minutes=$(date +%M)
if [ $heures -ge 19 ] && [ $heures -lt 20 ] && [ $minutes -lt 30 ] ; then
    avertir "C'est l'heure du bain !" "$(date +%R)" face-smirk
fi

# Ecrit l'heure dans un fichier nommé d'après la date du jour :
echo $(date +%X) >> $fichier
# Compte le nombre de lignes de ce fichier :
periodes=$(wc -l < $fichier)
# Nombre maximum de périodes de cinq minutes autorisées :
limite=25
if [ $periodes -ge $limite ] ; then
    avertir "L'ordinateur va s'arrêter dans quatre minutes !" "Attention !" face-angry
    # Il faut laisser le temps (4 minutes) à l'administrateur de désactiver le script si jamais la session enfants s'ouvre automatiquement :
    sudo shutdown -h +4
    # Au redémarrage pour désactiver cette commande : sudo shutdown -c puis déconnexion de la session
elif [ $periodes -ge $(($limite-1)) ] ; then
    avertir "Vous avez fait trop d'ordinateur !" "Il va s'arrêter dans 9 minutes !" face-angry
else
    # Liste des processus à surveiller (ici 7 périodes de cinq minutes autorisées) :
    for p in "supertuxkart" "supertux2" ; do 
        surveiller $p 7
    done
fi
